/*:
 # Reduce playground
 
 Map, filter, and reduce examples.
 
 Also, a simple implementation of `map` using `reduce`.
 
 From [https://swiftunboxed.com/lang/reduce](https://swiftunboxed.com/lang/reduce)
*/
import Foundation

//: Map
let words = ["hello", "there"]
let shoutyWords = words.map { $0.uppercased() }
print(shoutyWords)

//: Filter
let numbers = 0...10
let evenNumbers = numbers.filter { $0 % 2 == 0 }
print(evenNumbers)

//: Reduce
let primes = [3, 7, 31, 127]
let sum = primes.reduce(0) { (result, item) in
  return result + item
}
print(sum)

//: Building map using reduce
extension Array {
  func mapUsingReduce<OutElement>(
    _ closure: (_ item: Iterator.Element) -> OutElement)
    -> [OutElement] {
      return self.reduce([]) { (partial, item) in
        return partial + [closure(item)]
      }
  }
}

let shoutyWordsAgain = words.mapUsingReduce { $0.uppercased() }
print(shoutyWordsAgain)

//: Building filter using reduce
extension Sequence {
  func filterUsingReduce(
    _ closure: (_ item: Iterator.Element) -> Bool)
    -> [Iterator.Element] {
      return self.reduce([]) { (partial, item) in
        return partial + (closure(item) ? [item] : [])
      }
  }
}

let oddNumbers = numbers.filterUsingReduce { $0 % 2 == 1 }
print(oddNumbers)
